# ********************************************************************************
# +----------------------------------------------------------------------------
# | File : Publish.ps1    updated 10/31  
# +----------------------------------------------------------------------------
# ********************************************************************************

<#
.SYNOPSIS
Publish script to identify the location of App-V Acrobat on system, deploy Acrobat App-V in various Licensing modes, and suppress the Updater on the client.

.DESCRIPTION
.\Publish.ps1 [[-PackageGUID] <GUID>] [[-RelativePath] <Acrobat's Path>] [-CreateLog] [-LogFile <Path to Logfile>] [-Serialize] [-ChannelName {"DC|2015|2017"}]

.EXAMPLE
.\Publish.ps1 daa6e80e-2e0f-4071-8d26-ba62b96c613d "[{AppVPackageRoot}]\Acrobat\Acrobat.exe" -Serialize -ChannelName "2015|2017"
Runs Publish.ps1 for Classic track and volume serializes Acrobat 

.EXAMPLE
.\Publish.ps1 abc6e80e-2d0f-4r71-8d59-ba62b96c613d "[{AppVPackageRoot}]\Acrobat\Acrobat.exe" -ChannelName "DC"
Runs Publish.ps1 for Continuous track and does not serialize Acrobat

.EXAMPLE
.\Publish.ps1 -PackageGUID daa6e80e-2e0f-4071-8d26-ba62b96c613d -RelativePath "[{AppVPackageRoot}]\Acrobat\Acrobat.exe" -CreateLog -Serialize -LogFile testresults.log -ChannelName "2015|2017"
Runs Publish.ps1 for Classic track, creates log inside testresults.log file and volume serializes Acrobat
#>

param (
#The package ID for the package on which the script will run.
[String]$PackageGUID, 
#Location of Acrobat.exe inside the package.
[String]$RelativePath, 
#Switched parameter given to Volume Serialize the Application on the client machine.
#If this parameter is given, make sure a copy of prov.xml is placed beside Publish.ps1.
[Switch]$Serialize, 
#switched parameter given to apply FRL license only package.
#if this is given, please ensure to add licensing tool kit and json file from FRL license only package.
[Switch]$Frl,
#Identifies the delivery track of the application, should match the track from which package is created. Value is either "DC" or "2015" or "2017".
[String]$ChannelName="2015",
#Specifies the filepath to which script logs will be written.
[String]$LogFile="$env:Temp\Acrobat-Appv.log", 
#Switched parameter that generates Script Logs on the client machine, at the location described by -LogFile parameter.
[Switch]$CreateLog)
# +---------------------------------------------------------------------------
# | FUNCTIONS
# | 
# | description of function, parameters and what it returns (if any)                                                                                         
# +----------------------------------------------------------------------------
function ShowHeading {
##-----------------------------------------------------------------------
## Function: ShowHeading
## Purpose: Use to write a header for the script
##-----------------------------------------------------------------------
        Log "---------------------------------------------------------"  
        Log "-                       Publish.ps1                     -"
        Log "-       Post Publishing Script for Acrobat App-V        -"
        Log "---------------------------------------------------------" 
}
function Log {
##-----------------------------------------------------------------------
## Function: Log
## Purpose: Used to write output to a log file
##-----------------------------------------------------------------------
    param ([string]$argOut)
    $kDateTime = get-date -uFormat "%d-%m-%Y %H:%M"
    $kOut = ($kDateTime+ ": " + $argOut)
    If ($LogFile -and $CreateLog) {"$kOut" | out-file -filepath $LogFile -append}
}
function PrintAndLog {
##-----------------------------------------------------------------------
## Function: PrintAndLog
## Purpose: Used to write output to the screen
##-----------------------------------------------------------------------
    param ([string]$argOut)
            write-host $argOut
    If ($CreateLog) {Log $argOut}
}
##---------------------------------------------------------------------------------
## Function: CreateAppVRegistry
## Purpose: This function is used to create appv key for our custom MSI to function
##---------------------------------------------------------------------------------
function CreateAppVRegistry
{
    param([string]$argIntegrationRoot, [string]$argInstallPath)
    PrintAndLog "-------------------------------------------------------------------------------"
    PrintAndLog "Setting appv registry key"
    PrintAndLog "-------------------------------------------------------------------------------"
    PrintAndLog "DEBUG: Checking $argInstallPath is available or not"
    If (Test-Path $argInstallPath) {
        # Registries to be made in Native Hive
        $kAcrobatHive = "HKLM:\Software\Adobe\Adobe Acrobat\$ChannelName"
        $kAcrobatAppvHive = $kAcrobatHive + "\appv"
        If (Test-Path $kAcrobatAppvHive){
            PrintAndLog "DEBUG: [Native] - $kAcrobatAppvHive is available"
        }
		Else {
            New-Item -Path $kAcrobatHive -Name appv -Force
        }
        Set-ItemProperty -Path $kAcrobatAppvHive -Name "INTEGRATION_ROOT" -Value $argIntegrationRoot
        Set-ItemProperty -Path $kAcrobatAppvHive -Name "INSTALL_PATH" -Value $argInstallPath
           
        # + Registries to be made in Wow Hive
        If([System.Environment]::Is64BitProcess){
            $kAcrobatWow64Hive = "HKLM:\Software\Wow6432Node\Adobe\Adobe Acrobat\$ChannelName"
            $kAcrobatWow64AppvHive = $kAcrobatWow64Hive + "\appv"
            If (Test-Path $kAcrobatWow64AppvHive) {
                PrintAndLog "DEBUG: [Wow6432] - $kAcrobatWow64AppvHive is available"
            } Else {
                New-Item -Path $kAcrobatWow64Hive -Name appv -Force
            }        
            Set-ItemProperty -Path $kAcrobatWow64AppvHive -Name "INTEGRATION_ROOT" -Value $argIntegrationRoot
            Set-ItemProperty -Path $kAcrobatWow64AppvHive -Name "INSTALL_PATH" -Value $argInstallPath
        }
    } 
}
##---------------------------------------------------------------------------------
## Function: PerformLicensingSetup
## Purpose: This function creates the necessary folders on the client machine with
##          required permissions and also places files in them.
##---------------------------------------------------------------------------------
function PerfromLicensingSetup
{
    param([string]$argProgramFilesCommon, [string]$argInstallPath)
    If($ChannelName -eq "2015") {
        $kAcrobatLEID = "V7{}AcrobatESR-12-Win-GM"      # For Classic 2015 Retail and Volume Serials
    }
    ElseIf($ChannelName -eq "2017") {
        $kAcrobatLEID = "V7{}AcrobatESR-17-Win-GM"      # For Classic 2017 Retail and Volume Serials
    }
	ElseIf($ChannelName -eq "2020") {
        $kAcrobatLEID = "V7{}AcrobatESR-20-Win-GM"      # For Classic 2020 Retail and Volume Serials
    }
    Else {
        $kAcrobatLEID = "V7{}AcrobatCont-12-Win-GM"     # For Continuous Retail and Volume Serials
    }
    $kPayloadCode = "{AC76BA86-1033-FFFF-7760-000000000007}"
    
    PrintAndLog "-------------------------------------------------------------------------------"
    PrintAndLog "Setting Licensing Related Folders with proper permissions "
    PrintAndLog "-------------------------------------------------------------------------------"
    PrintAndLog "DEBUG: Creating the following folders :-"
    
    # Getting folders setup correctly in Program Data
    If (!(Test-Path "C:\ProgramData\Adobe"))         { New-Item -ItemType directory -Path "C:\ProgramData\Adobe" }
    If (!(Test-Path "C:\ProgramData\Adobe\SLStore")) { New-Item -ItemType directory -Path "C:\ProgramData\Adobe\SLStore" }
    PrintAndLog "DEBUG: 1. SLStore - Comlpleted"
        
    # Getting folders setup correclty in $ProgramFilesCommon
    $kCommonFilesAdobe = $argProgramFilesCommon + "\Adobe"
    
    If (!(Test-Path $kCommonFilesAdobe)) { New-Item -ItemType directory -Path $kCommonFilesAdobe }
    If (!(Test-Path "$kCommonFilesAdobe\Adobe PCD")) { New-Item -ItemType directory -Path "$kCommonFilesAdobe\Adobe PCD" }
    PrintAndLog "DEBUG: 2. Adobe PCD - Comlpleted"
    
    If (!(Test-Path "$kCommonFilesAdobe\PCF"))       { New-Item -ItemType directory -Path "$kCommonFilesAdobe\PCF"       }
    PrintAndLog "DEBUG: 3. PCF - Comlpleted"
    
    If (!(Test-Path "$kCommonFilesAdobe\SLCache"))   { New-Item -ItemType directory -Path "$kCommonFilesAdobe\SLCache"   }
    PrintAndLog "DEBUG: 4. SLCache - Comlpleted"
    
    Copy-Item "$argInstallPath\AMT\application.xml" "$kCommonFilesAdobe\PCF\$kPayloadCode.$kAcrobatLEID.xml" -Force
    PrintAndLog "DEBUG: $argInstallPath\application.xml Files copied locally as $kCommonFilesAdobe\PCF\$kPayloadCode.$kAcrobatLEID.xml"
    
    cmd.exe /c "icacls.exe `"$kCommonFilesAdobe\Adobe PCD`" /grant Everyone:(M)"
    cmd.exe /c "icacls.exe `"$kCommonFilesAdobe\SLCache`" /grant Everyone:(M)"
    PrintAndLog "DEBUG: Permissions set correctly"
    
    PrintAndLog "DEBUG: Licensing Folders setup completed"
}
##---------------------------------------------------------------------------------
## Function: LicenseAcrobat
## Purpose: This function is responsible for Volume Serializing Acrobat on Client
##---------------------------------------------------------------------------------
function LicenseAcrobat
{
    PrintAndLog "-------------------------------------------------------------------------------"
    PrintAndLog " Serializing Acrobat with Volume Serial "
    PrintAndLog "-------------------------------------------------------------------------------"
    cmd.exe /c "adobe_prtk.exe --tool=VolumeSerialize --provfile=prov.xml --stream"
    PrintAndLog "DEBUG: Licensing Script execution completed" # Log Data
    PrintAndLog "-------------------------------------------------------------------------------"
}
##---------------------------------------------------------------------------------
## Function: ApplyFRL
## Purpose: This function is responsible for running FRL tool kit
##---------------------------------------------------------------------------------
function ApplyFRL
{
    PrintAndLog "-------------------------------------------------------------------------------"
    PrintAndLog " Applying FRL license "
    PrintAndLog "-------------------------------------------------------------------------------"
    cmd.exe /c "adobe-licensing-toolkit.exe -p -i -f ngl-preconditioning-data.json"
    PrintAndLog "DEBUG: FRL license applied" # Log Data
    PrintAndLog "-------------------------------------------------------------------------------"
}

##---------------------------------------------------------------------------------
## Function: SuppressUpdater
## Purpose: This function is responsible for Suppressing Updater in App-V Acrobat
##---------------------------------------------------------------------------------
function SuppressUpdater
{
    PrintAndLog "-------------------------------------------------------------------------------"
    PrintAndLog "Disabling Updater for Acrobat, so that it does not update through ARM"
    PrintAndLog "-------------------------------------------------------------------------------"
            
    # Registries to be made in the native hive
    $kAcrobatPoliciesHive = "HKLM:\SOFTWARE\Policies\Adobe\Adobe Acrobat\$ChannelName"
    $kAcrobatLockDownHive = $kAcrobatPoliciesHive + "\FeatureLockDown"
    If(Test-Path $kAcrobatLockDownHive) {
        PrintAndLog "DEBUG: [Native] - FeatureLockDown Key available"
    } Else {
        New-Item -Path $kAcrobatPoliciesHive -Name FeatureLockDown -Force
    }
    Set-ItemProperty -Path $kAcrobatLockDownHive -Name "bUpdater" -Value 0  # Disabling Updating of the product.
        
    If([System.Environment]::Is64BitProcess) {
        # + Registries to be made in the Wow6432 Hive
        $kAcrobatPoliciesWow6432Hive = "HKLM:\SOFTWARE\Wow6432Node\Policies\Adobe\Adobe Acrobat\$ChannelName"
        $kAcrobatLockDownWow6432Hive = $kAcrobatPoliciesHive + "\FeatureLockDown"
        If(Test-Path $kAcrobatLockDownWow6432Hive) {
            PrintAndLog "DEBUG: [Wow6432] - FeatureLockDown Key available"
        } Else {
            New-Item -Path $kAcrobatPoliciesWow6432Hive -Name FeatureLockDown -Force
        }
        Set-ItemProperty -Path $kAcrobatLockDownWow6432Hive -Name "bUpdater" -Value 0  # Disabling Updating of the product.
    }
}
##---------------------------------------------------------------------------------------
## Function: SuppressNotifications
## Purpose: This function is responsible for Suppressing Notifications in App-V Acrobat
##---------------------------------------------------------------------------------------
function SuppressNotifications
{
    PrintAndLog "-------------------------------------------------------------------------------"
    PrintAndLog "Disabling Notifications for Acrobat, as it is not supported in AppV"
    PrintAndLog "-------------------------------------------------------------------------------"
            
    # Registries to be made in the native hive
    $kAcrobatPoliciesHive = "HKLM:\SOFTWARE\Policies\Adobe\Adobe Acrobat\$ChannelName"
    $kAcrobatLockDownHive = $kAcrobatPoliciesHive + "\FeatureLockDown"
    If(Test-Path $kAcrobatLockDownHive) {
        PrintAndLog "DEBUG: [Native] - FeatureLockDown Key available"
    } Else {
        New-Item -Path $kAcrobatPoliciesHive -Name FeatureLockDown -Force
    }
	$kAcrobatServicesLockDownHive = $kAcrobatLockDownHive + "\cServices"
    If(Test-Path $kAcrobatServicesLockDownHive) {
        PrintAndLog "DEBUG: [Native] - Services Key available"
    } Else {
        New-Item -Path $kAcrobatLockDownHive -Name cServices -Force
    }
    Set-ItemProperty -Path $kAcrobatServicesLockDownHive -Name "bToggleNotifications" -Value 1  # Disabling Notifications of the product.
        
    If([System.Environment]::Is64BitProcess) {
        # + Registries to be made in the Wow6432 Hive
        $kAcrobatPoliciesWow6432Hive = "HKLM:\SOFTWARE\Wow6432Node\Policies\Adobe\Adobe Acrobat\$ChannelName"
		$kAcrobatLockDownWow6432Hive = $kAcrobatPoliciesWow6432Hive + "\FeatureLockDown"
		If(Test-Path $kAcrobatLockDownWow6432Hive) {
			PrintAndLog "DEBUG: [Native] - FeatureLockDown Key available"
		} Else {
			New-Item -Path $kAcrobatPoliciesWow6432Hive -Name FeatureLockDown -Force
		}
	    $kAcrobatServicesLockDownWow6432Hive = $kAcrobatLockDownWow6432Hive + "\cServices"
        If(Test-Path $kAcrobatServicesLockDownWow6432Hive) {
            PrintAndLog "DEBUG: [Native] - Services Key available"
        } Else {
            New-Item -Path $kAcrobatLockDownWow6432Hive -Name cServices -Force
        }
        Set-ItemProperty -Path $kAcrobatServicesLockDownWow6432Hive -Name "bToggleNotifications" -Value 1  # Disabling Notifications of the product.
    }
}
##---------------------------------------------------------------------------------
## Function: AppendToPassThrough
## Purpose: This function appends to App-V Passthrough keys appropriately
##---------------------------------------------------------------------------------
function AppendToPassThrough
{
    PrintAndLog "-------------------------------------------------------------------------------"
    PrintAndLog "Adding the necessary entries in PassThroughPaths key of App-V"
    PrintAndLog "-------------------------------------------------------------------------------"
    
    $kPassThroughHive = "HKLM:\SOFTWARE\Microsoft\AppV\Subsystem\VirtualRegistry"
    $kPassThroughKey = "PassThroughPaths"
    $arrPassThroughEntriesPresent = (Get-ItemProperty $kPassThroughHive $kPassThroughKey).$kPassThroughKey
	If(($ChannelName -eq "2015") -or ($ChannelName -eq "2017")) {
		$arrPassThroughEntriesToAdd = "HKEY_CURRENT_USER\Software\Microsoft\Internet Explorer\MenuExt"
	} Else {
		$arrPassThroughEntriesToAdd = "HKEY_CURRENT_USER\Software\Microsoft\Internet Explorer\MenuExt", "HKEY_CURRENT_USER\Software\Adobe\Adobe Acrobat\$ChannelName\DiskCabs", "HKEY_CURRENT_USER\Software\Adobe\Adobe Acrobat", "HKEY_CURRENT_USER\Software\Adobe\Acrobat Distiller", "HKEY_CURRENT_USER\Software\Adobe\Adobe Synchronizer", "HKEY_CURRENT_USER\Software\Adobe\CommonFiles", "HKEY_CURRENT_USER\Software\Adobe\PDF Settings", "HKEY_CURRENT_USER\Software\Microsoft\Speech", "HKEY_CURRENT_USER\Software\Lotus\Notes"
	}
    
    PrintAndLog "`nDEBUG: The entries available presently in passthrough are" #$arrPassThroughEntriesPresent"
    PrintAndLog $arrPassThroughEntriesPresent
    PrintAndLog "`nDEBUG: The entries to be added in passthrough are" #$arrPassThroughEntriesToAdd"
    PrintAndLog $arrPassThroughEntriesToAdd
    
    ForEach($Entry in $arrPassThroughEntriesToAdd){
        if($arrPassThroughEntriesPresent -notcontains $Entry){
            $arrPassThroughEntriesPresent += $Entry
        }
    }
    PrintAndLog "`nDEBUG: The entries to be present in passthrough are" #$arrPassThroughEntriesPresent"
    PrintAndLog $arrPassThroughEntriesPresent
    Set-ItemProperty $kPassThroughHive $kPassThroughKey -value $arrPassThroughEntriesPresent -type MultiString
    PrintAndLog "`nDEBUG: The keys successfully added to App-V PassThroughPaths"
}   
##-----------------------------------------------------------------------
## Function: Usage
## Purpose: This function is used to describe how to use this script
##-----------------------------------------------------------------------
function Usage
{
PrintAndLog
@"
    Usage: 
        
     .\Publish.ps1 -PackageGUID GUID -RelativePath Acrobat's Path [-CreateLog] [-LogFile <Path to Logfile>] [-Serialize] [-Frl] [-ChannelName {"DC|2015|2017|2020"}]
     Runs the Publish script to identify the location of App-V Acrobat on system, deploy Acrobat App-V in various Licensing modes, and suppress the Updater on the client.
     e.g. .\Publish.ps1 -PackageGUID daa6e80e-2e0f-4071-8d26-ba62b96c613d -RelativePath "[{AppVPackageRoot}]\Acrobat\Acrobat.exe" -CreateLog -Serialize -LogFile testresults.log -ChannelName "2015|2017|2020"
        
"@
    return
}
# +---------------------------------------------------------------------------
# | MAIN ROUTINE                                                                                         
# +----------------------------------------------------------------------------
function Main
{
    param (
    [string]$PackageGUID, 
    [string]$RelativePath, 
    [bool]$Serialize, 
    [bool]$Frl,
    [string]$ChannelName,
    [string]$LogFile,
    [string]$CreateLog)
    If (Test-Path $LogFile) {Remove-Item -path $LogFile}
    ShowHeading
    If (($PackageGUID -eq $null) -or ($PackageGUID -eq '') -or ($RelativePath -eq $null) -or ($RelativePath -eq '')) 
    {
        Usage
        return
    }
    
    $kIntegrationRootGlobal = (Get-AppvClientConfiguration | where {$_.Name -eq "IntegrationRootGlobal"}).Value
    If ($kIntegrationRootGlobal) {
        $kIntegrationRootGlobal = [System.Environment]::ExpandEnvironmentVariables($kIntegrationRootGlobal)
        $kIntegrationRoot = [System.IO.Path]::Combine($kIntegrationRootGlobal, $PackageGUID + "\Root\")
        $kPhysicalPath = $RelativePath.Replace('\Acrobat.exe', '')  # This provides the physical path of Acrobat which will need to be updated while patching.
        
        # Given the fact that the package will be made following our guidelines the location of Acrobat EXE should be As follows :-
        # [AppVPackageIntegrationRoot]
        # |_
        #   Acrobat
        #   |_
        #     Acrobat.exe
        # Since, we already have constructed the Integration Root above, appending "Acrobat" to that path should be enough for the Install Path.
        $kInstallPath = [System.IO.Path]::Combine($kIntegrationRoot, "Acrobat")
        
        PrintAndLog "DEBUG: Application available physically at $kInstallPath"
        PrintAndLog "DEBUG: Package Integrated at $kIntegrationRootGlobal"
        
        # Step 1: Perform the registry creation (appv + policies)
        CreateAppVRegistry $kIntegrationRoot $kInstallPath
        AppendToPassThrough
        SuppressUpdater
        SuppressNotifications
        
        #Step 2: Perform Licensing Workflows
        If ([System.Environment]::Is64BitProcess) {
            $kProgramFiles = "C:\Program Files (x86)"
        } Else {
            $kProgramFiles = "C:\Program Files"
        }
        PerfromLicensingSetup "$kProgramFiles\Common Files" $kPhysicalPath
        If($Serialize) 
        {
            LicenseAcrobat
        }
	If($Frl) 
        {
	    PrintAndLog "DEBUG : FRL Enabled"
            ApplyFRL
        }
    } Else {
        PrintAndLog "DEBUG: Package Installation Root Global could not be found"
        return
    }
}

Main $PackageGUID $RelativePath $Serialize $Frl $ChannelName $LogFile $CreateLog 
